#pragma once

#include "Color.h"
#include "Transform.h"

#include <string>

class SceneItem
{
public:

    enum class Type
    {
        UNSET,
        MODEL,
        TEXT
    };

    SceneItem();

    virtual ~SceneItem();

    const Color& getFillColor() const;

    const Color& getOutlineColor() const;

    const Transform& getTransform() const;

    virtual Type getType() const = 0;

    double getOutlineThickness() const;

    const std::string& getName() const;

    bool hasFillColor() const;

    bool hasOutlineColor() const;

    bool isVisible() const;

    void setIsVisible(bool isVisible);

    void setOutlineColor(const Color& color);

    void setOutlineThickness(double thickness);

    void setFillColor(const Color& color);

    void setName(const std::string& name);

    void updateTransform(const Transform& transform);

protected:
    Transform mTransform;

    bool mHasFillColor{ false };
    Color mFillColor;

    bool mHasOutlineColor{ false };
    Color mOutlineColor;
    double mOutlineThickness{ 1.0 };

    bool mColorIsDirty{true};
    bool mTransformIsDirty{true};
    bool mIsVisible{true};

    std::string mName;
};
